package com.vanheusden.pfa;

import java.util.*;

class AlphaBeta {
	private int alpha, beta;
	private Move move;
	private List<Move> path;
	private boolean cutOff = false, pathAdded = false;

	public AlphaBeta(final AlphaBeta abIn) {
		this.alpha = abIn.getAlpha();
		this.beta = abIn.getBeta();
	}

	public AlphaBeta(final int alpha, final int beta) {
		this.alpha = alpha;
		this.beta = beta;
	}

	public AlphaBeta() {
		this.alpha = -Brain.value_infinite;
		this.beta = Brain.value_infinite;
	}

	public AlphaBeta(final int score, final boolean co) {
		this.alpha = score;
		this.beta = Brain.value_infinite;
		cutOff = co;
	}

	public AlphaBeta(final Move m, final int score, final List<Move> path, final boolean co) {
		this.alpha = score;
		this.move = m;
		this.path = path;
		this.cutOff = co;
	}

	void setMove(final Move m) {
		this.move = m;
	}

	// returns true for cut-off
	public boolean update(final AlphaBeta objsIn) {
		final Move moveIn = objsIn.getMove();

		final int score = -objsIn.getEval();

		if (score > alpha) {
			alpha = score;
			move = moveIn;
			path = objsIn.getPath();

			if (score >= beta) {
				// the PV (member "path") should be set to null if (score >= beta)
				// since in that case it is not really a PV. PVs correspond to
				// scores > alpha and < beta. For PV display purposes this becomes
				// relevant at the root.
				path = null;
				cutOff = true;
				return true;
			}
		}

		return false;
	}

	public boolean update(final Flag flag, final int score) {
		if (flag == Flag.LOWER) {
			if (score > alpha)
				alpha = score;
		}
		else /* if (flag == Flag.UPPER) */ {
			if (score < beta)
				beta = score;
		}

		if (alpha >= beta) {
			cutOff = true;
			return true;
		}

		return false;
	}

	public int getAlpha() {
		return alpha;
	}

	public int getEval() {
		return alpha; //!
	}

	public int getBeta() {
		return beta;
	}

	public boolean getCutOff() {
		return cutOff;
	}

	public List<Move> getPath() {
		if (!pathAdded) {
			if (path == null)
				path = new ArrayList<Move>();

			path.add(0, move);

			pathAdded = true;
		}

		return path;
	}

	public AlphaBeta getResult() {
		if (move == null)
			return null;

		if (path == null)
			path = new ArrayList<Move>();

		path.add(0, move);

		return this;
	}

	public Move getMove() {
		return move;
	}

	public String toString() {
		return "" + move + "|" + alpha + "/" + beta + "#" + cutOff;
	}

	public MoveResult getMoveResult(final int depth, final String source) {
		String pathStr = "";

		List<Move> p = getPath();

		if (p != null) {
			for(final Move cur : p)
				pathStr += cur + " ";
		}

		return new MoveResult(getEval(), depth, pathStr, getMove(), source);
	}
}
