package com.vanheusden.pfa;

import java.io.*;
import java.sql.*;
import java.util.*;

public class BookBestMove extends Book {
	public BookBestMove(final String url) throws Exception {
		super(url);

		if (table == null)
			table = "entries";
	}

	public MoveResult search(final Scene s, final Move lastMove, final PlayerColor who) throws Exception {
		MoveResult selectedMove = null;

		try {
			List<Move> moveList = s.getMoveList(who);
			if (moveList.isEmpty()) {
				Statics.log("BookBestMove: movelist is empty");
				return null;
			}

			PlayerColor otherSide = Statics.opponentColor(who);

			Map<String, Move> map = new HashMap<String, Move>();

			StringBuilder query = new StringBuilder("SELECT fen, pv, eval, plies FROM " + table + " WHERE fen IN (");
			boolean first = true;

			for(Move currentMove : moveList) {
				Scene sa = s.Move(currentMove);
				final Board b = sa.getBoard();
				final String fen = Statics.toFENString(b, otherSide, currentMove, 0, true);

				map.put(fen, currentMove);

				if (first)
					first = false;
				else
					query.append(", ");

				query.append("'" + fen + "'");
			}

			query.append(") AND NOT eval IS NULL ORDER BY eval ASC");

			Statics.log("Query: " + query.toString());

			ResultSet getRS = get.executeQuery(query.toString());

			int n = 0;
			while(getRS.next()) {
				Move currentMove = map.get(getRS.getString("fen"));

				if (currentMove == null) {
					Statics.log("BookBestMove: fen not found in map");
					continue;
				}

				int currentScore = -getRS.getInt("eval"); /* '-' because opponent score */

				if (selectedMove == null || currentScore > selectedMove.getEval())
					selectedMove = new MoveResult(currentScore, getRS.getInt("plies"), getRS.getString("pv"), currentMove, getName());

				n++;
			}

			Statics.log("BookBestMove: " + n + " hits");

			if (n == 0)
				Statics.log("BookBestMove: no result set");

			Statics.log("BookBestMove: " + selectedMove);
		}
		catch(Exception e) {
			throw new Exception(getName() + ": " + e);
		}

		return selectedMove;
	}
}
