package com.vanheusden.pfa;

import java.io.*;
import java.util.*;
import java.util.concurrent.*;

class BookThread implements Runnable {
	Thread t;
	Book b;

	final Object notifyStart = new Object();
	final Object notifyEnd = new Object();

	final Semaphore lock = new Semaphore(1, false);
	Scene s;
	Move lastMove;
	PlayerColor who;

	MoveResult br;

	public BookThread(final Book b) {
		this.b = b;

		br = null;

		t = new Thread(this);
		t.start();
	}

	public void setupSearch(final Scene s, final Move lastMove, final PlayerColor who) {
		lock.acquireUninterruptibly();
		this.s = s;
		this.lastMove = lastMove;
		this.who = who;
		br = null;
		lock.release();

		synchronized(notifyStart) {
			notifyStart.notify();
		}
	}

	public void run() {
		Statics.log("BookThread started for " + b.getName());

		try {
			for(;;) {
				try {
					synchronized(notifyStart) {
						notifyStart.wait();
					}
				}
				catch(InterruptedException ie) {
				}

				lock.acquireUninterruptibly();
				if (s != null) {
					br = b.search(s, lastMove, who);

					synchronized(notifyEnd) {
						notifyEnd.notify();
					}

					s = null;
				}
				lock.release();
			}
		}
		catch(Exception e) {
			Statics.logException("BookThread", e);
		}
	}

	MoveResult getResult() {
		for(;;) {
			lock.acquireUninterruptibly();
			boolean finished = s == null;
			lock.release();

			if (finished)
				break;

			try {
				synchronized(notifyEnd) {
					notifyEnd.wait();
				}
			}
			catch(InterruptedException ie) {
			}
		}

		lock.acquireUninterruptibly();
		MoveResult brOut = br;
		lock.release();

		return brOut;
	}
}
