package com.vanheusden.pfa;

// maximum material value (1 king against *)
// 8 * 100 + 2 * 500 + 2 * 325 + 2 * 325 + 975 = 4075

enum ChessObjectType { PAWN(0, "p", (byte)5, 100), ROOK(2, "r", (byte)2, 500), KNIGHT(1, "n", (byte)4, 325), BISHOP(1, "b", (byte)3, 325), QUEEN(3, "q", (byte)1, 975), KING(7, "k", (byte)0, 10000);
	final int value, evalVal;
	final byte svi;
	final String name;

	ChessObjectType(final int value, final String name, final byte svi, final int evalVal) {
		this.value = value;
		this.name = name;
		this.svi = svi;
		this.evalVal = evalVal;
	}

	// used for move-ordening
	final int getValue() {
		return value;
	}

	final String getName() {
		return name;
	}

	// for zobrist hashing
	final byte getSVI() {
		return svi;
	}

	final static byte getSVINullType() {
		return 6;
	}

	// for evaluation of position
	final int getEvalVal() {
		return evalVal;
	}

	static final ChessObjectType getCOT(byte svi) {
		switch(svi) {
			case 0:
				return KING;
			case 1:
				return QUEEN;
			case 2:
				return ROOK;
			case 3:
				return BISHOP;
			case 4:
				return KNIGHT;
			case 5:
				return PAWN;
			case 6:
				return null;
		}

		return null;
	}
}
