package com.vanheusden.pfa;

import java.io.*;
import java.net.*;
import java.security.SecureRandom;
import java.util.*;

abstract class IO {
	abstract void loop() throws IOException, InterruptedException;
	protected String pgnFile = null;
	final SecureRandom r = new SecureRandom();
	//
	protected double maxDuration = 5.0;
	int maxDepth = 4;
        int maxExtension = 5;
        boolean random = true;
	PlayerColor currentPlayer = PlayerColor.WHITE;
	Scene scene = null;
	Brain brain = null;
	BrainPonder bp = null;
	boolean ponder = true;
	MoveRepetition moveRepHist = new MoveRepetition();
	Pgn pgnObject = null;
	boolean ignore_protocol_overrides = false;
	List<String> beepers = null;

	public IO(final int maxDepth, final int maxExtension, final boolean random, final double maxDuration, final Brain brain, final boolean ponder, final boolean ignore_protocol_overrides, final List<String> beepers) {
		this.maxDepth = maxDepth;
		this.maxExtension = maxExtension;
		this.random = random;
		this.maxDuration = maxDuration;
		this.brain = brain;
		this.ponder = ponder;
		this.ignore_protocol_overrides = ignore_protocol_overrides;
		this.beepers = beepers;

		if (brain != null)
			brain.setIO(this);

		if (ponder && !(brain instanceof BrainPuppetMaster))
			bp = new BrainPonder(this, brain);
	}

	public void exception(Exception e) {
		try {
			progressCallback("Exception: " + e);
			progressCallback("Details: " + e.getMessage());
			progressCallback("Thread: " + Thread.currentThread().getName());
			progressCallback("ArrayDeque-trace:");

			for(StackTraceElement ste: e.getStackTrace())
			{
					progressCallback(" " + ste.getClassName() + ", "
						+ ste.getFileName() + ", "
						+ ste.getLineNumber() + ", "
						+ ste.getMethodName() + ", "
						+ (ste.isNativeMethod() ?
							"is" : "NOT a") + " native method");
			}
		}
		catch(IOException ie) {
			System.err.println("Exception during exception " + ie);
		}
	}

	public void progressCallbackMateAt(int distance) {
	}

	public void progressCallbackWindowSize(int alpha, int beta) {
	}

	public void progressThreadStats(int cntRunnable, int cntBlocked, int slotsUsage, int slotsInUse, long nThreadsCreated) {
	}

	public void processChessStats(long nExtensions, long nTransTab, long nCutOffs, long nQueiesceNodes) {
	}

	public void progressTTLockDistribution(double sd) {
	}

	public void progressCallbackXboardShowThinking(int id, int score, long durationCentiSeconds, long nodesVisited, String details) throws IOException {
		String what = "" + id + " " + score + " " + durationCentiSeconds + " " + nodesVisited + " " + details;

		Statics.log(what);
	}

	public void progressCallback(String what) throws IOException {
		Statics.log(what);
	}

	public void progressCallbackDebug(String what) throws IOException {
		Statics.log(what);
	}

	public void progressBenchmark(int depth, int nThreads, int numberOfThreadSlots, long nNodesVisited, double took, Board b, MoveResult m, long nExtensions, long nCutOffs, long nTPT) throws IOException {
		String what = "" + depth + " " + nThreads + " " + numberOfThreadSlots + " " + nNodesVisited + " " + took + " " + (nNodesVisited / took) + " " + nExtensions + " " + nCutOffs + " " + nTPT + " " + m.toString();

		Statics.log(what);
	}

	public void progressCallbackNPS(int nps) throws IOException {
		if (Statics.verbose > 0)
			Statics.log("Nodes/s " + nps);
	}

	public void progressCallbackTimeSearch(int ms) throws IOException {
		Statics.log("Time searched: " + ms + "ms");
	}

	public void progressCallbackBest(Move move, int score) throws IOException {
		Statics.log("Best move: " + move + " (" + score + ")");
	}

	public void progressCallbackNNodes(long nNodes) throws IOException {
		if (Statics.verbose > 0)
			Statics.log("Total nodes searched: " + nNodes);
	}

	public void progressCallbackDepthReached(int depth) throws IOException {
		Statics.log("Depth reached: " + depth);
	}

	public void progressCallbackCurrMove(Move move) throws IOException {
		Statics.log("Current move: " + move);
	}

	void beepDo(final String address) throws IOException { // FIXME move to io-class
		// 192.168.59.1
		// bfreq length
		try {
			DatagramSocket clientSocket = new DatagramSocket();

			InetAddress IPAddress = InetAddress.getByName(address);

			String sentence = "b666 250";
			byte[] sendData = sendData = sentence.getBytes();

			DatagramPacket sendPacket = new DatagramPacket(sendData, sendData.length, IPAddress, 34478);
			clientSocket.send(sendPacket);

			clientSocket.close();
		}
		catch(Exception e) {
			progressCallback("" + e);
		}
	}

	public void beep() throws IOException {
		if (beepers == null)
			return;

		for(final String ip : beepers) {
			try {
				beepDo(ip);
			}
			catch(IOException ie) {	
				progressCallback("" + ie);
			}
		}
	}
}
