package com.vanheusden.pfa;

import java.io.*;
import java.util.*;

class IO_Test extends IO {
	private int prevDepth = -1;

	public void progressCallback(String what) {
		Statics.log(what);
		System.out.println(what);
	}

	public void progressCallbackDebug(String what) {
		if (Statics.verbose > 0) {
			Statics.log(what);
			System.out.println(what);
		}
	}

	public void progressCallbackNPS(int nps) throws IOException {
		Statics.log("Nodes/s " + nps);
		System.out.println("Nodes/s " + nps);
	}

	public void progressCallbackDepthReached(int depth) throws IOException {
		if (depth != prevDepth) {
			String what = "Depth reached: " + depth + " (thread: " + Thread.currentThread().getName() + ")";
			Statics.log(what);
			System.out.println(what);

			prevDepth = depth;
		}
	}

	public IO_Test(String pgnFile, int maxDepth, int maxExtension, boolean random, double maxDuration, final Brain brain, final String boardInit, final boolean ponder, List<String> beepers) throws Exception {
		super(maxDepth, maxExtension, random, maxDuration, brain, ponder, false, beepers);

		if (boardInit != null) {
			Object [] epd = Statics.EPDToScene(boardInit);
			scene = (Scene)epd[0];
			currentPlayer = (PlayerColor)epd[1];
		}
		else {
			scene = new Scene();
		}

		this.pgnFile = pgnFile;
	}

	public void loop() throws IOException, InterruptedException {
		int it = 0;
		Pgn pgnObject = null;
		if (pgnFile != null)
			pgnObject = new Pgn(PlayerColor.WHITE, pgnFile);

		Move lastMove = null;

		for(;;) {
			MoveResult move = null;

			if (currentPlayer == PlayerColor.WHITE) {
				it++;

				System.out.println("Move #" + it + " WHITE");
			}
			else {
				System.out.println("Move #" + it + " BLACK");
			}

			scene.validateMoves(currentPlayer);

			if (scene.isStaleMate(PlayerColor.WHITE)) {
				System.out.println("White is stale-mate");

				if (pgnObject != null)
					pgnObject.closePgnFile("1/2-1/2");
				break;
			}

			if (scene.isStaleMate(PlayerColor.BLACK)) {
				System.out.println("Black is stale-mate");

				if (pgnObject != null)
					pgnObject.closePgnFile("1/2-1/2");
				break;
			}

			if (scene.isCheckMate(PlayerColor.WHITE)) {
				System.out.println("White is check-mate");

				if (pgnObject != null)
					pgnObject.closePgnFile("0-1");
				break;
			}

			if (scene.isCheckMate(PlayerColor.BLACK)) {
				System.out.println("Black is check-mate");

				if (pgnObject != null)
					pgnObject.closePgnFile("1-0");
				break;
			}

			if (scene.isInsufficientMaterialDraw()) {
				System.out.println("Not enough objects to win");

				if (pgnObject != null)
					pgnObject.closePgnFile("1/2-1/2");
				break;
			}

			Statics.displayBoard(scene.getBoard());

			move = brain.calculateMove(it, scene, maxDepth, currentPlayer, moveRepHist, (long)(maxDuration * 1000.0), 0, lastMove, false);

			if (Statics.getVerbosity() > 0) {
				System.out.println(move.toString());
			}

			if (pgnObject != null)
				pgnObject.pgnEmitMove(scene.getBoard(), move.getMove(), currentPlayer);

			if (move.getMove() == null) {
				System.out.println("" + currentPlayer + " gives up");
				break;
			}

			Move m = move.getMove();

			System.out.println("Will move: " + m + " (" + move.getEval() + ")");

			lastMove = m;

			moveRepHist.push(currentPlayer, brain.getHash(scene.getBoard(), currentPlayer));
			scene = scene.Move(m);

			currentPlayer = Statics.opponentColor(currentPlayer);
		}
	}
}
