package com.vanheusden.pfa;

import java.io.*;
import java.util.*;

class IO_TestANSI extends IO_ConsoleANSI {
	public IO_TestANSI(String pgnFile, int maxDepth, int maxExtension, boolean random, double maxDuration, final Brain brain, final String boardInit, final boolean ponder, List<String> beepers) throws Exception {
		super(pgnFile, maxDepth, maxExtension, random, maxDuration, boardInit, brain, ponder, beepers);

		if (boardInit != null) {
			Object [] epd = Statics.EPDToScene(boardInit);
			scene = (Scene)epd[0];
			currentPlayer = (PlayerColor)epd[1];
		}
		else {
			scene = new Scene();
		}

		this.pgnFile = pgnFile;
	}

	public void loop() throws IOException, InterruptedException {
		int it = 0;
		Pgn pgnObject = null;
		if (pgnFile != null)
			pgnObject = new Pgn(PlayerColor.WHITE, pgnFile);

		initTerminal();

		Move lastMove = null;
		HistoryElement he = null;

		for(;;) {
			term_reset_attributes();
			term_set_fg_color(termColorWhite);
			term_move_cursor(1, 1);

			if (currentPlayer == PlayerColor.WHITE) {
				it++;

				he = new HistoryElement();

				term_move_cursor(xDataCol1, 1);
				term_set_fg_color(termColorRed);
				System.out.print("move " + it);
			}

			scene.validateMoves(currentPlayer);

			if (checkCheckMate(PlayerColor.WHITE) || checkCheckMate(PlayerColor.BLACK) ||
				checkStaleMate(PlayerColor.WHITE) || checkStaleMate(PlayerColor.BLACK) ||
				checkInsufficientMaterialDraw()) {
				break;
			}

			displayBoard(scene.getBoard(), lastMove);

			String fen = Statics.toFENString(scene.getBoard(), currentPlayer, lastMove, it, false);

			term_move_cursor(1, yInput);
			term_clear_line();

			term_set_fg_color(termColorBlue);
			term_blink_slow();
			System.out.print(" * DBP " + currentPlayer + " is calculating * ");
			term_reset_attributes();
			term_set_fg_color(termColorWhite);

			MoveResult move = brain.calculateMove(it, scene, maxDepth, currentPlayer, moveRepHist, (long)(maxDuration * 1000.0), 0, lastMove, false);

			String out = Statics.moveToString(move.getMove(), false) + " (" + move.getEval() + ")";
			Statics.log("" + it + ". POS moves: " + out);

			if (currentPlayer == PlayerColor.WHITE) {
				he.setWhite(fen, move.getMove());
				h.add(he);
			}
			else {
				he.setBlack(fen, move.getMove());

				he = new HistoryElement();
			}

			showMoveList();

			if (pgnObject != null)
				pgnObject.pgnEmitMove(scene.getBoard(), move.getMove(), currentPlayer);

			if (move.getMove() == null) {
				System.out.println("" + currentPlayer + " gives up");
				break;
			}

			if (pgnObject != null)
				pgnObject.pgnEmitMove(scene.getBoard(), move.getMove(), currentPlayer);

			moveRepHist.push(currentPlayer, brain.getHash(scene.getBoard(), currentPlayer));

			scene = scene.Move(move.getMove());

			double score = move.getEval();
			putSmiley(xDataCol1, 7, score, previousScore);
			previousScore = score;

			currentPlayer = Statics.opponentColor(currentPlayer);
		}
	}
}
