package com.vanheusden.pfa;

import java.io.*;
import java.net.*;

class IO_UCI_socket_android extends IO_UCI {
	ServerSocket ss = null;
	PosService ps = null;
	Socket client = null;
	
	public void progressCallback(String what) {
		if (Statics.verbose) {
			emit("info string " + what);
			sendMessage(what);
		}
	}

	public void progressCallbackDebug(String what) {
		emit("info string " + what);
		sendMessage(what);
	}

	public void sendMessage(String what) {
		ps.sendMessage(what);
	}

	IO_UCI_socket_android(PosService ps, String pgnFile, int maxDepth, int maxExtension, boolean random, double maxDuration, int port, final Brain brain, final boolean ponder, List<String> beepers) throws Exception {
		super(pgnFile, maxDepth, maxExtension, random, maxDuration, brain, ponder, false, beepers);

		this.ps = ps;

		ss = new ServerSocket(port);
		ss.setReuseAddress(true);
	}
	
	void close() {
		try {
			if (xboardPipe != null)
				xboardPipe.close();

			if (output != null)
				output.close();

			if (client != null)
				client.close();
		}
		catch(IOException ie) {
			System.err.println("IO_UCI_socket: close failed " + ie);
		}
		
		xboardPipe = null;
		output = null;
		client = null;
	}

	void setupIO() throws IOException {
	}

	public int getPort() {
		return ss.getLocalPort();
	}

	void reconnect() {
		System.out.println("IO_UCI_socket::reconnect()");

		for (;;) {
			try {
				client = ss.accept();
				sendMessage("Connected with " + client.getRemoteSocketAddress());

				client.setTcpNoDelay(true);

				xboardPipe = new BufferedReader(new InputStreamReader( client.getInputStream()));

				output = new BufferedWriter(new OutputStreamWriter( client.getOutputStream()));

				break;
			} catch (IOException ie) {
				System.err.println("IO_UCI_socket::reconnect " + ie);
			}
		}
	}

	synchronized void emit(String what) {
		if (output == null) // should not occur
			reconnect();
		
		try {
			// no newLine because then the 'what' could succeed with the
			// newLine() failing causing a duplicate 'what' after reconnect
			output.write(what + "\n");
			output.flush();

			System.out.println("socket out: " + what);

		} catch (IOException ie) {
			System.err.println("IO_UCI_socket::emit " + ie);

			close();
		}
	}

	String get() {
		if (xboardPipe == null)
			reconnect();

		try {
			String in = xboardPipe.readLine();

			System.out.println("socket in: " + in);

			return in;
		} catch (IOException ie) {
			System.err.println("IO_UCI_socket::get " + ie);

			close();

			return "quit";
		}
	}
}
