package com.vanheusden.pfa;

import java.io.*;
import java.util.*;

class IO_XBoard extends IO {
	/* private stuff */
	BufferedReader xboardPipe = new BufferedReader(new InputStreamReader(System.in));
	PlayerColor myColor = PlayerColor.BLACK;
	boolean doNotCalculateMovesSelf = false;
	Move myLastMove = null;
	boolean commercialSent = false;
	boolean gameStarted = false, moveEmitted = false;
	Hashtable<String, String> namesKnown = new Hashtable<String, String>();
	boolean onIcs = false;
	Pgn pgnObject = null;
	int it = 0;
	Move lastMove = null;
	boolean xboardAllowPonder = true;
	List<History> history = new ArrayList<History>();

	final int defaultNMovesPerGame = 40;

	int maxMoves = defaultNMovesPerGame;
	int movesLeft = maxMoves;
	double timeLeft = 5.0, maxTime = 10;
	int timeInc = 0;
	//
	boolean isFixedTime = false;
	double fixedTime = -1.0;

	long calcStartTs = 0;
	AsyncBrain ab = null;

	public void progressCallback(String what) throws IOException {
		// emit("1 +0 0 0 " + what);
		Statics.log(what);
	}

	public void progressCallbackDebug(String what) throws IOException {
		// if (Statics.verbose > 0)
		// 	emit("1 +0 0 0 " + what);
		Statics.log(what);
	}

	public void progressCallbackXboardShowThinking(int id, int score, long durationCentiSeconds, long nodesVisited, String details) throws IOException {
		if (Statics.verbose > 0)
			emit("" + id + " " + score + " " + durationCentiSeconds + " " + nodesVisited + " " + details);
	}

	synchronized void emit(String what) {
		System.out.println(what);
		System.out.flush();

		Statics.log("Out: <" + what + ">");
	}

	private void editBoard() throws IOException {
		List<ChessObject> white = new ArrayList<ChessObject>();
		List<ChessObject> black = new ArrayList<ChessObject>();
		PlayerColor currentColor = PlayerColor.WHITE;

		ChessObject [][] board = new ChessObject[8][8]; /* y */

finished_editing_board:
		for(;;) {
			String line = xboardPipe.readLine();

			if (line.compareTo(".") == 0) {
				break finished_editing_board;
			}
			else if (line.compareTo("c") == 0) {
				currentColor = Statics.opponentColor(currentColor);
			}
			else if (line.compareTo("#") == 0) {
				board = new ChessObject[8][8]; /* y */
			}
			else if (line.charAt(0) == 'x') {
				int x = line.charAt(1) - 'a';
				int y = line.charAt(2) - '1';

				board[y][x] = null;
			}
			else if (line.length() == 3) {
				int x = line.charAt(1) - 'a';
				int y = line.charAt(2) - '1';

				ChessObjectType type = Statics.CharToChessObjectType(line.charAt(0));

				if (type == null) {
					sendError(line, null);
				}
				else {
					Statics.log("Putting a " + type + " with color " + currentColor + " at position " + x + "," + y + " (" + line + ")");
					board[y][x] = ChessObject.newChessObject(type, x, y, currentColor);
				}
			}
			else {
				sendError(line, null);
			}
		}

		/* now create a new set of objects for this board */
		boardToObjectLists(board, white, black);

		restartGame(new Scene((ChessObject [])white.toArray(), (ChessObject [])black.toArray()), false);
	}

	void boardToObjectLists(ChessObject [][] board, List<ChessObject> white, List<ChessObject> black) {
		for(int y=0; y<8; y++) {
			for(int x=0; x<8; x++) {
				if (board[y][x] != null) {
					if (board[y][x].getColor() == PlayerColor.WHITE) {
						white.add(board[y][x]);
					}
					else if (board[y][x].getColor() == PlayerColor.BLACK) {
						black.add(board[y][x]);
					}
				}
			}
		}
	}

	private void restartGame(Scene newScene, boolean resetForceMode) throws IOException {
		scene = newScene;

		if (scene == null)
			scene = new Scene();

		scene.validateMoves(currentPlayer);

		moveRepHist = new MoveRepetition();

		if (resetForceMode)
			doNotCalculateMovesSelf = false;

		currentPlayer = PlayerColor.WHITE;
		myColor = PlayerColor.BLACK;

		myLastMove = null;
		lastMove = null;

		commercialSent = false;

		if (pgnObject != null && moveEmitted == true) {
			gameStarted = false;
			moveEmitted = false;
			pgnObject.closePgnFile("*");
		}

		isFixedTime = false;
		fixedTime = -1.0;

		it = 1;
	}

	private void sendError(String cmd, String par) {
		String error;

		if (par != null)
			error = "Command " + cmd + " " + par + " not understood.";
		else
			error = "Command " + cmd + " not understood.";

		Statics.log(error);
		emit("tellusererror " + error);
	}

	void switchUser() {
		currentPlayer = Statics.opponentColor(currentPlayer);
		if (currentPlayer == PlayerColor.WHITE)
			it++;
	}

	void doEngineMove() throws IOException, InterruptedException {
		double currentDuration = 0.1;

		// taken from fairymax 4.8q-2
		if (isFixedTime) {
			currentDuration = fixedTime;
			Statics.log("Fixed time, duration: " + currentDuration);
		}
		else {
			int curNMoves = movesLeft <= 0 ? defaultNMovesPerGame : movesLeft;
			currentDuration = 0.6 * (timeLeft + (curNMoves - 1) * timeInc) / (curNMoves + 7);

			double limitDuration = timeLeft / 15.0;
			if (currentDuration > limitDuration)
				currentDuration = limitDuration;

			if (currentDuration > maxDuration && maxDuration > 0) {
				Statics.log("Selected think-time longer than max (" + currentDuration + "), limiting to " + maxDuration);
				currentDuration = maxDuration;
			}

			Statics.log("curNMoves: " + curNMoves + ", current duration: " + currentDuration + ", timeLeft: " + timeLeft + ", limitDuration: " + limitDuration);
		}

		// //

		calcStartTs = System.currentTimeMillis();

		stopPonder();

		ab.startAsyncBrain(it, scene, 0, currentPlayer, moveRepHist, (long)(currentDuration * 1000.0), 0, lastMove);
	}

	MoveResult getCalcResult() throws IOException, InterruptedException {
		if (!ab.tryStopSearch(100))
			return null;

		MoveResult move = ab.getResult();
		if (move == null)
			return null;

		if (move.getMove() != null)
			pgnObject.pgnEmitMove(scene.getBoard(), move.getMove(), currentPlayer);
		moveEmitted = true;

		Scene prevScene = scene;
		scene = scene.Move(move.getMove());

		moveRepHist.push(currentPlayer, brain.getHash(prevScene.getBoard(), currentPlayer));

		switchUser();

		scene.validateMoves(currentPlayer);

		if (bp != null && ponder && (xboardAllowPonder || ignore_protocol_overrides))
			bp.startPonder(it, scene, maxDepth, currentPlayer, moveRepHist, 0, 0, move.getMove());

		String moveStr = Statics.moveToString(move.getMove(), true);
		emit("move " + moveStr);
		Statics.log("(pos) move " + it + ": " + moveStr + ", value: " + move.getEval());

		long endTs = System.currentTimeMillis();

		if (!isFixedTime) {
			// fairymax
			timeLeft -= (endTs - calcStartTs) / 1000.0;
			timeLeft += timeInc;

			movesLeft--;
			if (movesLeft == 0) {
				movesLeft = maxMoves;

				if (maxMoves == 1)
					timeLeft = maxTime;
				else
					timeLeft += maxTime;
			}
			// //
		}

		calcStartTs = 0;

		return move;
	}

	boolean checkGameEnd() throws IOException {
		if (scene.isCheckMate(PlayerColor.WHITE)) {
			emit("0-1 {Black mates}");
			if (pgnObject != null)
				pgnObject.closePgnFile("0-1");
			return true;
		}
		if (scene.isCheckMate(PlayerColor.BLACK)) {
			emit("1-0 {White mates}");
			if (pgnObject != null)
				pgnObject.closePgnFile("1-0");
			return true;
		}

		if (scene.isStaleMate(PlayerColor.WHITE)) {
			emit("0-1 {stale mate caused by black}");
			if (pgnObject != null)
				pgnObject.closePgnFile("0-1");
			return true;
		}
		if (scene.isStaleMate(PlayerColor.BLACK)) {
			emit("1-0 {stale mate caused by white}");
			if (pgnObject != null)
				pgnObject.closePgnFile("1-0");
			return true;
		}

		return false;
	}

	void sendCommercial(String name) {
		if (!commercialSent && onIcs) {
			commercialSent = true;

			if (name != null && namesKnown.contains(name) == false) {
				namesKnown.put(name, "");

				emit("tellopponent You're playing against " + Version.getIdentifier() + " - http://www.vanheusden.com/DeepBrutePos/ - written by folkert@vanheusden.com");
			}
		}
	}

	void startPgnFileWrapper(PlayerColor posColor) {
		if (gameStarted == false) {
			gameStarted = true;
			pgnObject = new Pgn(posColor, pgnFile);
		}
	}

	IO_XBoard(String pgnFile, int maxDepth, int maxExtension, boolean random, double maxDuration, final Brain brain, final boolean ponder, final boolean ignore_protocol_overrides, List<String> beepers) throws IOException {
		super(maxDepth, maxExtension, random, maxDuration, brain, ponder, ignore_protocol_overrides, beepers);

		this.pgnFile = pgnFile;

		if (maxDuration > 0)
			maxTime = maxDuration;

		ab = new AsyncBrain(this, brain);

		restartGame(null, true);

		if (bp != null)
			bp.startPonder(it, scene, maxDepth, currentPlayer, moveRepHist, 0, 0, null);
	}

	void stopPonder() throws IOException {
		if (bp != null)
			bp.stopPonder();

		brain.stopPonder();
	}

	public void loop() throws IOException, InterruptedException {
		String opponentName = "unknown";
		String opponentHost = "localhost";

		scene.validateMoves(currentPlayer);

		Statics.log(Statics.startupLine);
		Statics.log("" + Version.getName() + " started in xboard mode");

		brain.setName(System.getProperty("user.name") + "@localhost");

		for(;;) {
			String cmd = null, par = null;
			String line = xboardPipe.readLine();
			if (line == null) {
				Statics.log("Got null!");
				break;
			}

			Statics.log("In: <" + line + ">");

			if (line.length() == 0)
				continue;

			String [] pars = null;
			int space = line.indexOf(' ');
			if (space == -1) {
				cmd = line;
			}
			else {
				cmd = line.substring(0, space);
				par = line.substring(space + 1);
				pars = par.split(" ");
			}

			if (cmd.compareTo("quit") == 0) {
				stopPonder();
				break;
			}
			else if (cmd.compareTo("xboard") == 0) {
				emit("");		/* this lets xboard know we're xboard-ready */
			}
			else if (cmd.compareTo("protover") == 0) {
				emit("feature name=1 ping=1 san=1 sigint=0 sigterm=0 analyze=0 ics=1 setboard=1 smp=1 playother=0 myname=\"" + Version.getIdentifier() + "\" done=1");	/* no special features required */
			}
			else if (cmd.compareTo("setboard") == 0) {
				Object [] epd = Statics.EPDToScene(pars[0] + " " + pars[1] + " " + pars[2] + " " + pars[3]);
				scene = (Scene)epd[0];
				currentPlayer = (PlayerColor)epd[1];

				Statics.logBoard(scene.getBoard());
			}
			else if (cmd.compareTo("book") == 0) {
				// FIXME
			}
			else if (cmd.compareTo("accepted") == 0) {
				// FIXME
			}
			else if (cmd.compareTo("new") == 0) {
				stopPonder();

				restartGame(null, true);
				timeLeft  = maxTime;
				movesLeft = maxMoves;
				isFixedTime = false;
			}
			else if (cmd.compareTo("variant") == 0) {
				// FIXME now how should i handle this?
			}
			else if (cmd.compareTo("random") == 0) {
				/* this allows the engine to more or less choose randomly moves */
				brain.setAllowRandom(true);

				Statics.log("Random is enabled.");
			}
			else if (cmd.compareTo("force") == 0) {		// just accept moves, don't do anything yourself
				doNotCalculateMovesSelf = true;
			}
			else if (cmd.compareTo("go") == 0) {		// exit force-mode
				doNotCalculateMovesSelf = false;

				myColor = currentPlayer;
				startPgnFileWrapper(myColor);

				doEngineMove();
				myLastMove = null;

				for(;;) {
					if (System.in.available() > 0) {
						Statics.log("*** SEARCH ABORTED DUE TO INPUT FROM STDIN ***");
						ab.abortSearch();
						calcStartTs = 0;
						break;
					}

					MoveResult temp = getCalcResult();
					if (temp != null) {
						myLastMove = temp.getMove();
						break;
					}
				}

				while(maxMoves > 0 && movesLeft <= 0)
					movesLeft += maxMoves;

				/* check mate? */
				checkGameEnd();
			}
			else if (cmd.compareTo("white") == 0) {
				currentPlayer = PlayerColor.WHITE;
				myColor = PlayerColor.BLACK;
				doNotCalculateMovesSelf = false;		// 20060921 15:47 
			}
			else if (cmd.compareTo("black") == 0) {
				currentPlayer = PlayerColor.BLACK;
				myColor = PlayerColor.WHITE;
				doNotCalculateMovesSelf = false;		// 20060921 15:47 
			}
			else if (cmd.compareTo("level") == 0) {
				isFixedTime = false;
				if (pars != null && pars.length == 3) {
					maxMoves = movesLeft = Integer.valueOf(pars[0]);

					if (pars[1].indexOf(":") != -1) {
						String [] fields = pars[1].split(":");

						timeLeft = maxTime = Integer.valueOf(fields[0]) * 60 + Integer.valueOf(fields[1]);
					}
					else {
						timeLeft = maxTime = Integer.valueOf(pars[1]) * 60;
					}

					timeInc = Integer.valueOf(pars[2]);

					Statics.log("Moves: " + maxMoves + ", maxTime: " + maxTime + ", timeInc: " + timeInc);
				}
				else {
					emit("tellopponent level command missing parameter(s)");
				}
			}
			else if (cmd.compareTo("cores") == 0) {
				if (pars != null) {
					if (!ignore_protocol_overrides)
						brain.setNCores(Integer.valueOf(par));
				}
				else {
					emit("tellopponent cores command missing parameter");
				}
			}
			else if (cmd.compareTo("st") == 0) {
				if (pars != null) {
					fixedTime = Double.valueOf(par);
					isFixedTime = true;
				}
				else {
					emit("tellopponent st command missing parameter");
				}
			}
			else if (cmd.compareTo("sd") == 0) {
				if (pars != null)
					maxDepth = Integer.valueOf(par);
				else
					emit("tellopponent sd command missing parameter");
			}
			else if (cmd.compareTo("cores") == 0) {
				if (pars != null)
					brain.setNCores(Integer.valueOf(par));
				else
					emit("tellopponent cores command missing parameter");
			}
			else if (cmd.compareTo("time") == 0) {
				isFixedTime = false;
				if (pars != null)
					timeLeft = Integer.valueOf(par) / 100.0;
				else
					emit("tellopponent time command missing parameter");
			}
			else if (cmd.compareTo("otim") == 0) {
				// FIXME
			}
			else if (cmd.compareTo("?") == 0) {
				// FIXME: "move now" - we're allowed to ignore it
			}
			else if (cmd.compareTo("ping") == 0) {
				emit("pong " + par);
			}
			else if (cmd.compareTo("draw") == 0) {
				/* say we accept it but just continue playing (as specified in the xboard specs) */
				emit("offer draw");
			}
			else if (cmd.compareTo("result") == 0) {
				String resultStr = par;

				space = resultStr.indexOf(' ');
				if (space != -1)
					resultStr = resultStr.substring(0, space);

				stopPonder();
			}
			else if (cmd.compareTo("edit") == 0) {
				editBoard();
			}
			else if (cmd.compareTo("hint") == 0) {
				MoveResult move = brain.calculateMove(it, scene, maxDepth, Statics.opponentColor(myColor), moveRepHist, 5000l, 0, lastMove, false);

				if (move.getMove() != null)
					emit("Hint: " + Statics.moveToString(move.getMove(), true));
				else
					emit("Hint: no move");
			}
			else if (cmd.compareTo("bk") == 0) {
				// FIXME?
				emit("\tValues of beta may give rise to dom.");
				emit("");
			}
                        else if (cmd.compareTo("undo") == 0) {
                                if (history.size() > 0) {
					History h = history.remove(history.size() - 1);

					Object [] epd = Statics.EPDToScene(h.getFEN());
					scene = (Scene)epd[0];
					currentPlayer = (PlayerColor)epd[1];
                                }
                        }
                        else if (cmd.compareTo("remove") == 0) {
                                if (history.size() > 1) {
					// his opponent
					History h = history.remove(history.size() - 1);
					// himself
					h = history.remove(history.size() - 1);

					Object [] epd = Statics.EPDToScene(h.getFEN());
					scene = (Scene)epd[0];
					currentPlayer = (PlayerColor)epd[1];
                                }
                        }
			else if (cmd.compareTo("hard") == 0) {
				xboardAllowPonder = true;

				if (bp != null && ponder && (xboardAllowPonder || ignore_protocol_overrides))
					bp.startPonder(it, scene, maxDepth, currentPlayer, moveRepHist, 0, 0, lastMove);
			}
			else if (cmd.compareTo("easy") == 0) {
				xboardAllowPonder = false;

				if (!ignore_protocol_overrides)
					stopPonder();
			}
			else if (cmd.compareTo("post") == 0) {
			}
			else if (cmd.compareTo("nopost") == 0) {
			}
			else if (cmd.compareTo("analyze") == 0) {
				// FIXME
			}
			else if (cmd.compareTo("name") == 0) {
				opponentName = par;

				Statics.log("playing against: " + opponentName);
				brain.setName(opponentName + "@" + opponentHost);

				sendCommercial(opponentName);
			}
			else if (cmd.compareTo("rating") == 0) {
				Statics.log("ICS rating: " + par);
			}
			else if (cmd.compareTo("ics") == 0) {
				if (par.compareTo("-") != 0)
					onIcs = true;

				opponentHost = par;

				Statics.log("Opponent is on host: " + par);

				brain.setName(opponentName + "@" + opponentHost);
			}
			else if (cmd.compareTo("computer") == 0) {
				// FIXME
			}
			else {
				String moveString = cmd;

				if (((myLastMove != null && moveString.compareTo(Statics.moveToString(myLastMove, true)) != 0) || myLastMove == null)) {
					boolean moveValid = true;
					StringBuffer error = new StringBuffer("");
					Move move = Statics.SANToMove(moveString, scene.getBoard(), currentPlayer, scene.getMoveList(currentPlayer), error);

					if (move == null)
						lastMove = move = Statics.LANToMove(moveString, scene.getBoard(), currentPlayer, scene.getMoveList(currentPlayer), error);

					if (move != null) {
						Statics.log("got move " + it + " from opponent: " + move);

						history.add(new History(Statics.toFENString(scene.getBoard(), currentPlayer, lastMove, 1, false), move));

						lastMove = move;

						sendCommercial(opponentName);

						startPgnFileWrapper(myColor);

						if (scene.isMoveInMovelist(move, currentPlayer)) {
							moveRepHist.push(currentPlayer, brain.getHash(scene.getBoard(), currentPlayer));

							pgnObject.pgnEmitMove(scene.getBoard(), move, currentPlayer);
							moveEmitted = true;
							scene = scene.Move(move);
							switchUser();
							scene.validateMoves(currentPlayer);

							/* check mate? */
							if (checkGameEnd() == false) {
								if (doNotCalculateMovesSelf == false) {
									assert currentPlayer == myColor;

									history.add(new History(Statics.toFENString(scene.getBoard(), currentPlayer, lastMove, 1, false), move));
									/* calculate move by engine */
									doEngineMove();
									lastMove = myLastMove = null;

									for(;;) {
										if (System.in.available() > 0) {
											Statics.log("*** SEARCH ABORTED DUE TO INPUT FROM STDIN ***");
											ab.abortSearch();
											calcStartTs = 0;
											break; // FIXME check if `force' or `new'
										}

										MoveResult temp = getCalcResult();
										if (temp != null) {
											lastMove = myLastMove = temp.getMove();
											break;
										}
									}

									/* check mate? */
									checkGameEnd();
								}

								if (it >= 75) { // offer draw if >= 75 moves (not half moves)
									Statics.log("offer draw after " + it + " moves");
									emit("offer draw");
								}
							}
						}
						else {
							moveValid = false;
						}
					}
					else {
						moveValid = false;
					}

					if (!moveValid) {
						String errorMessage = "Illegal move (" + error + "): " + moveString;
						emit(errorMessage);
						Statics.log(errorMessage);

						Statics.log("Current player: " + currentPlayer);
						Statics.log("Pos color: " + myColor);

						Statics.log("recv move: " + move);

						Statics.logBoard(scene.getBoard());

						Statics.log("valid moves:");
						for(Move currentMove : scene.getMoveList(currentPlayer)) {
							Statics.log(" " + currentMove);
						}
					}
				}
				else {
					Statics.log("ignoring my own move");
				}
			}
		}

		Statics.log("---exit---" );
	}
}
