package com.vanheusden.pfa;

import java.io.*;
import java.util.*;

class Pgn {
	private String pgnBuffer = new String();
	private String pgnFile;

	class PgnMovePair {
		String white = null, black = null;

		PgnMovePair(final String white) {
			this.white = white;
		}

		void setBlack(final String black) {
			this.black = black;
		}

		String getWhite() {
			return white;
		}

		String getBlack() {
			return black;
		}
	}

	private PgnMovePair workPair = null;
	private List<PgnMovePair> moves = new ArrayList<PgnMovePair>();

	private void emitPgnLine(String line) {
		pgnBuffer += line + "\n";
	}

	public Pgn(final PlayerColor posColor, final String pgnFile) {
		String white, black;

		this.pgnFile = pgnFile;

		if (posColor == PlayerColor.WHITE) {
			white = Version.getIdentifier();
			black = "opponent";
		}
		else {
			white = "opponent";
			black = Version.getIdentifier();
		}

		init(white, black);
	}

	public Pgn(final String white, final String black, final String pgnFile) {
		this.pgnFile = pgnFile;

		init(white, black);
	}

	void init(final String white, final String black) {
		emitPgnLine("[Event \"" + white + " versus " + black + "\"]");

		emitPgnLine("[Site \"?\"]");
		Calendar rightNow = Calendar.getInstance();
		emitPgnLine("[Date \"" + rightNow.get(Calendar.YEAR) + "." + (rightNow.get(Calendar.MONTH) + 1) + "." + rightNow.get(Calendar.DAY_OF_MONTH) + "\"]");
		emitPgnLine("[Round \"1\"]");
		emitPgnLine("[White \"" + white + "\"]");
		emitPgnLine("[Black \"" + black + "\"]");
	}

	public void pgnEmitMove(Board board, Move move, PlayerColor color) {
		String moveString = "";

		if (move.getMoveType() == MoveType.CASTLING && move.getTX() == 6) {
			moveString += "O-O";
		}
		else if (move.getMoveType() == MoveType.CASTLING && move.getTX() == 2) {
			moveString += "O-O-O";
		}
		else {
			ChessObjectType cot = board.getAt(move.getFX(), move.getFY()).getType();
			String objectType = cot == ChessObjectType.PAWN ? "" : Statics.objectTypeToString(cot, true);

			String promoteTo = move.getPromoteTo() != null ? Statics.objectTypeToString(move.getPromoteTo(), true) : null;

			// add object type
			moveString += objectType;
			// which position from
			moveString += (char)('a' + move.getFX());
			moveString += (char)('1' + move.getFY());
			// add 'x' if it is a capture move
			if (board.getAt(move.getTX(), move.getTY()) != null)
				moveString += 'x';
			// add move itself
			moveString += (char)('a' + move.getTX());
			moveString += (char)('1' + move.getTY());
			// promotion to?
			if (promoteTo != null)
				moveString += "=" + promoteTo;
		}

		if (color == PlayerColor.WHITE) {
			workPair = new PgnMovePair(moveString);

			moves.add(workPair);
		}
		else {
			if (workPair == null)
				workPair = new PgnMovePair("?");

			workPair.setBlack(moveString);
		}
	}

	void storeFile(String what) throws IOException {
		FileWriter file = new FileWriter(pgnFile, true);
		PrintWriter pw = new PrintWriter(file);
		pw.println(what);
		pw.flush();
		pw.close();
	}

	public void closePgnFile(String result) throws IOException {
		if (moves.size() > 0) {
			emitPgnLine("[Result \"" + result + "\"]");
			emitPgnLine("");

			int nr = 1;
			String out = "";
			for(PgnMovePair cur : moves) {
				if (cur.getBlack() == null) {
					out += "" + nr + "." + cur.getWhite() + " ";
					break;
				}

				out += "" + nr + "." + cur.getWhite() + " " + cur.getBlack() + " ";

				nr++;
			}

			out += result;

			emitPgnLine(out);
		}

		if (pgnFile != null && pgnBuffer.length() > 0)
			storeFile(pgnBuffer);
	}
}
