package com.vanheusden.pfa;

import java.lang.Thread;
import java.lang.Thread.State;
import java.util.*;

public class Profiler implements Runnable {
	private Map<String, Long> map = new HashMap<String, Long>();
	private List<Integer> runningCount = new ArrayList<Integer>();
	private Thread t = null;
	private boolean filterJava = false;
	private int interval = 10;

	public Profiler(final boolean filterJava, final int interval) {
		this.filterJava = filterJava;
		this.interval = interval;

		t = new Thread(this);
		t.start();
	}

	public void stopThread() {
		t.interrupt();
	}

	public Map<String, Long> getStatistics() {
		return map;
	}

	public void dump() {
		long total = 0;
		for(String key : map.keySet())
			total += map.get(key);

		for(String key : map.keySet()) {
			Long value = map.get(key);

			System.err.println("MCOUNT\t" + key + "\t" + value + "\t" + (value * 100.0) / total);
		}

		for(int count : runningCount)
			System.err.println("RCOUNT\t" + count);
	}

	public void run() {
		for(;!Thread.interrupted();) {
			try {
				Thread.sleep(interval);
			}
			catch(InterruptedException ie) {
				break;
			}

			final Thread me = Thread.currentThread();

			final ThreadGroup tg = me.getThreadGroup();

			final Thread [] list = new Thread[1024];

			final int n = tg.enumerate(list);

			int nRunning = 0;
			for(int i=0; i<n; i++) {
				if (list[i].getId() == me.getId())
					continue;

				final Thread.State st = list[i].getState();

				if (st == State.RUNNABLE)
					nRunning++;

				if (st == State.NEW || st == State.TERMINATED)
					continue;

				final StackTraceElement [] sts = list[i].getStackTrace();
				for(int stIndex=0; stIndex<sts.length; stIndex++) {
					StackTraceElement ste = sts[stIndex];
					String className = ste.getClassName();

					if ((className.substring(0, 5).equals("java.") == false && className.substring(0, 4).equals("sun.") == false) || !filterJava) {
						String key = ste.getClassName() + "::" + ste.getMethodName();

						Long value = map.get(key);

						if (value == null)
							value = new Long(1);
						else
							value++;

						map.put(key, value);

						break;
					}
				}
			}

			runningCount.add(nRunning);
		}
	}
}
